							/****************************************************************************************************/
							/*								           PROJECT:-												*/
							/*								Ethernet Interface for Automations Systems        					*/
							/*															                            			*/
							/*																									*/				
							/*																									*/
							/****************************************************************************************************/
 
#define F_CPU 7372800UL
#include <avr/io.h>
#include <stdlib.h>
#include <string.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "ip_arp_udp_tcp.h"
#include "enc28j60.h"
#include "avr_compat.h"
#include "net.h"
#include "uart.h"
#include"lcd.h"
#include "i2cmaster.h"

#define UART_BAUD_RATE      9600

static uint8_t mymac[6] = {0x54,0x90,0x58,0x10,0x65,0x77};
static uint8_t myip[4] = {192,168,2,248};

#define MYWWWPORT 80
#define BUFFER_SIZE 850
static uint8_t buf[BUFFER_SIZE+1];
uint16_t packet;
char buffer[10];
char buffer2[10];

// prepare the webpage by writing the data to the tcp send buffer
uint16_t print_webpage(uint8_t *buf)
{
        uint16_t plen;
	    plen=fill_tcp_data_p(buf,0,PSTR("HTTP/1.0 200 OK\r\nContent-Type: text/html\r\nPragma: no-cache\r\n\r\n"));
		plen=fill_tcp_data_p(buf,plen,PSTR("<IMG SRC=\"https://lh5.googleusercontent.com/-8O1_4slsQ2M/T_yUdutSJeI/AAAAAAAAAM4/3voVXgAYdxw/s800/EFY-Group-Logo.jpg\" ALIGN=TOP>"));
		plen=fill_tcp_data_p(buf,plen,PSTR("<p> <h2><center>EFY AUTOMATION THROUGH WEBSERVER</center></h2><hr></p>"));
		plen=fill_tcp_data_p(buf,plen,PSTR("<iframe src=\"http://free.timeanddate.com/clock/n176/tlin/fn11/fs30/ftu/pa3/tt0/tw1/tm1\" frameborder=\"0\" width=\"340\" height=\"40\"></iframe> "));
		plen=fill_tcp_data_p(buf,plen,PSTR("<br><br><br><a href = \"../AUTOMATION\">TEMPERATURE:</a>"));
		plen=fill_tcp_data(buf,plen,buffer);
		plen=fill_tcp_data_p(buf,plen,PSTR("<sup>o</sup>C"));
		plen=fill_tcp_data_p(buf,plen,PSTR("<br><br><a href = \"../LUX-METER\">INTENSITITY OF LIGHT</a>"));
		plen=fill_tcp_data(buf,plen,buffer2);
		plen=fill_tcp_data_p(buf,plen,PSTR(" LUX"));
		plen=fill_tcp_data_p(buf,plen,PSTR("<IMG SRC=\"https://lh5.googleusercontent.com/-eJj4x7lL5Lc/T_yY6yXlAjI/AAAAAAAAANE/nOvCqHZ9v6M/s800/1.jpg\" ALIGN=RIGHT>"));
		plen=fill_tcp_data_p(buf,plen,PSTR("<br><br><br><br><hr><bottom right>powered by EFY..</bottom right>\n"));
		return(plen);

}

int main(void)
		{
        int i,j,p,q;
		long double ch0,ch1,lux;
		float ms=0.0;
		float s=0.0;
		float min=14;
		float hr=02;
		uint16_t plen;
        uint16_t dat_p;   
		
							
		lcd_init(LCD_DISP_ON);			//function to display on lcd
		uart1_init( UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU) );
		uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU) );
		sei();
		
		DDRA = 0xFF;
		PORTA = 0xFF;
		
		DDRD |= (1 <<7);

		uart1_puts("Board ON \r\n");
	    uart_puts("Board ON \r\n");
		
        /*initialize enc28j60*/
        enc28j60Init(mymac);
		_delay_ms(12);
        
        /* Magjack leds configuration, see enc28j60 datasheet, page 11 */
        // LEDB=yellow LEDA=green
        // 0x476 is PHLCON LEDA=links status, LEDB=receive/transmit
        // enc28j60PhyWrite(PHLCON,0b0000 0100 0111 01 10);		
        enc28j60PhyWrite(PHLCON,0x476);
		_delay_ms(12);
		
        //init the ethernet/ip layer:
        init_ip_arp_udp_tcp(mymac,myip,MYWWWPORT);
		
        while(1){
                // get the next new packet:
                plen = enc28j60PacketReceive(BUFFER_SIZE, buf);

                /*in case of plen=0 packet is invalid */
                if(plen==0){
                        continue;
                }
				
                // check if ip packets are for us:
                if(eth_type_is_ip_and_my_ip(buf,plen)==0){
                        continue;
                }
                
                if(buf[IP_PROTO_P]==IP_PROTO_ICMP_V && buf[ICMP_TYPE_P]==ICMP_TYPE_ECHOREQUEST_V){
                        make_echo_reply_from_request(buf,plen);
						uart1_puts("ping packet recieved \r\n");
                        continue;
                }
				
				
                // tcp port www start, compare only the lower byte
                if (buf[IP_PROTO_P]==IP_PROTO_TCP_V&&buf[TCP_DST_PORT_H_P]==0&&buf[TCP_DST_PORT_L_P]==MYWWWPORT){
                        if (buf[TCP_FLAGS_P] & TCP_FLAGS_SYN_V){
                                make_tcp_synack_from_syn(buf);
                                // make_tcp_synack_from_syn does already send the syn,ack
                                continue;
                        }
                        if (buf[TCP_FLAGS_P] & TCP_FLAGS_ACK_V){
                                init_len_info(buf);
                                dat_p=get_tcp_data_pointer();
                                if (dat_p==0){
                                        if (buf[TCP_FLAGS_P] & TCP_FLAGS_FIN_V){
                                                // finack, answer with ack
                                                make_tcp_ack_from_any(buf);
                                        }
                                        // just an ack with no data, wait for next packet
                                        continue;
								}
								
								for(packet=0 ; packet < BUFFER_SIZE ; packet++)
								{
											uart1_putc((buf[packet]) );
								
											if (strncmp( "AUTOMATION" , (char *)& buf[packet], 10) == 0)		//variation of temperature
										{			i2c_init();
		
													i2c_start(0b10011111); // sending address and Read bit
													i = i2c_readAck();
													
													i2c_stop();
													
													dtostrf(i,11,2,buffer);
													lcd_gotoxy(0,0);
													lcd_puts(buffer);
													_delay_ms(500);
													lcd_clrscr();
													
													if(i>=0b00110010)
													{
														PORTA=0b110110;
														PORTD &= ~(1 << 7);
														lcd_gotoxy(0,0);
														lcd_puts(buffer);
														lcd_gotoxy(12,0);
														lcd_putc(0b11011111);		//degree symbol corresponding to "LCD HD44780"
														lcd_putc('C');
														lcd_gotoxy(1,1);
														lcd_puts("its too hot");
													}
													if(i<=0b00010000)
													{
														PORTA=0b00101101;
														PORTD |= (1 << 7);
														lcd_gotoxy(0,0);
														lcd_puts(buffer);
														lcd_gotoxy(12,0);
		                                                lcd_putc(0b11011111);		//degree symbol corresponding to "LCD HD44780"
		                                                lcd_putc('C');
														lcd_gotoxy(1,1);
														lcd_puts("its too cold");
													}
													if(i>0b00010000 || i<0b00110010)
													{
														PORTA=0b00011011;
														lcd_gotoxy(0,0);
														lcd_puts(buffer);
														lcd_gotoxy(12,0);
		                                                lcd_putc(0b11011111);		//degree symbol corresponding to "LCD HD44780"
		                                                lcd_putc('C');
														lcd_gotoxy(1,1);
														lcd_puts("its normal temperature");
													}
										break;
										}
									if (strncmp( "LUX-METER" , (char *)& buf[packet], 9) == 0)
										{
										
										 
										  
													i2c_init();
		
													i2c_start(0b01010010);
													i2c_write(0b10000000);			
													i2c_write(0b00000011);				
													i2c_stop();
													
	
													i2c_start(0b01010010);		
													i2c_write(0b10001100);				
													i2c_rep_start(0b01010011);		
													i=i2c_readNak();			
			
													i2c_rep_start(0b01010010);		
													i2c_write(0b10001101);				
													i2c_rep_start(0b01010011);		
													j=i2c_readNak();			
		
													i2c_rep_start(0b01010010);		
													i2c_write(0b10001110);				
													i2c_rep_start(0b01010011);		
													p=i2c_readNak();	
		
													i2c_rep_start(0b01010010);		
													i2c_write(0b10001111);				
													i2c_rep_start(0b01010011);		
													q=i2c_readNak();			
		
													i2c_stop();
													ch0=(i+j*256);
													ch1=(p+q*256);
		
										if(ch1/ch0 >=0 && ch1/ch0 <.52)
													lux=(.0315*ch0)-(.0593*ch0*pow(ch1/ch0,1.4));		
										if(ch1/ch0 >=.52 && ch1/ch0 <.65)
													lux=(.0229*ch0)-(.0291*ch1);		
										if(ch1/ch0 >=.65 && ch1/ch0 <.80)
													lux=(.0157*ch0-.0180*ch1);	
										if(ch1/ch0 >=.80 && ch1/ch0 <1.3)
													lux=(.00338*ch0-.00260*ch1);						
										if(ch1/ch0 >=1.30 )
													lux=0;
					
										
										_delay_ms(3000);
										lcd_clrscr();
										dtostrf(lux,8,3,buffer2);
										lcd_puts(buffer2);		
										lcd_gotoxy(12,0);
										lcd_puts("LUX");
										
													if(lux>=0b00000010)
													{
														PORTA=0b11111111;
														
                                                        PORTD |= (1 << 7);
														lcd_gotoxy(5,1);
														lcd_puts("BULB OFF");
													}
													if(lux<=0b00000001)
													{
														PORTA=0b00000000;
														
														PORTD &= ~(1 << 7);
														
														lcd_gotoxy(5,1);
														lcd_puts("BULB ON");
													}
													
										break;
									}
								}
								
                                plen=print_webpage(buf); // prepare the page for sending
                                make_tcp_ack_from_any(buf); // send ack for http get
                                make_tcp_ack_with_data(buf,plen); // send data
                                continue;
                        }
                }
        }
        return (0);
}
